{*******************************************************}
{                                                       }
{                                                       }
{       Unit : EdgeDetectUtils                          }
{       Created by Tsviatko Jongov                      }
{       http://tsviatko.jongov.com                      }
{       tsviatko@jongov.com                             }
{                                                       }
{       Date : 10.11.2006                               }
{                                                       }
{       Video edge detect utils.                        }
{                                                       }
{*******************************************************}
unit EdgeDetectUtils;

interface

uses
  Windows, SysUtils, Math;

type
  TEdgeDetector = class
  private
    FBuffer1: Pointer;
    FBuffer2: Pointer;

    FSrc1  : PByte;
    FSrc2  : PByte;
    FSrc3  : PByte;
    FDst   : PByte;
  public
    constructor Create;
    destructor Destroy; override;

    function HorizontalEdgeRGB32(Data: Pointer; W, H: Integer; K13, K2: Double): Bool;
    function VerticalEdgeRGB32(Data: Pointer; W, H: Integer; K13, K2: Double): Bool;
    function HorVerEdgeRGB32(Data: Pointer; W, H: Integer; K13, K2: Double): Bool;
  end;

implementation

{------------------------------------ TEdgeDetector ---------------------------}
constructor TEdgeDetector.Create;
begin
  try
    //Buffers must be 10 MB so we can handle the HD uncompressed frames (1920 x 1080)
    FBuffer1 := GetMemory(10 * 1024 * 1024);
    FBuffer2 := GetMemory(10 * 1024 * 1024);
  except
    OutputDebugString('tsv: Error in TEdgeDetector.Create call...');
  end;
end;

destructor TEdgeDetector.Destroy;
begin
  try
    FreeMemory(FBuffer1);
    FreeMemory(FBuffer2);

    inherited;
  except
    OutputDebugString('tsv: Error in TEdgeDetector.Destroy call...');
  end;
end;

function TEdgeDetector.HorizontalEdgeRGB32(Data: Pointer; W, H: Integer; K13, K2: Double): Bool;
var
  I, U: Integer;
begin
  Result := False;
  try
    //If the picture is bigger than HD 1920 x 1080 recreate buffers
    if (W * H > 1920 * 1080) then
    begin
      FreeMemory(FBuffer1);
      FBuffer1 := GetMemory(W * H * 4);
    end;

    CopyMemory(FBuffer1, Data, W * H * 4);

    for U := 0 to H - 1 do
    begin
      FSrc1 := PByte(Integer(FBuffer1) + U * W * 4);
      FSrc2 := PByte(Integer(FBuffer1) + U * W * 4 + 4);
      FSrc3 := PByte(Integer(FBuffer1) + U * W * 4 + 8);
      FDst  := PByte(Integer(Data) + U * W * 4 + 4);

      for I := 0 to W - 3 do
      begin
        FDst^ := Trunc(K13 * FSrc1^ + K2 * FSrc2^ + K13 * FSrc3^);
        Inc(FSrc1);
        Inc(FSrc2);
        Inc(FSrc3);
        Inc(FDst);

        FDst^ := Trunc(K13 * FSrc1^ + K2 * FSrc2^ + K13 * FSrc3^);
        Inc(FSrc1);
        Inc(FSrc2);
        Inc(FSrc3);
        Inc(FDst);

        FDst^ := Trunc(K13 * FSrc1^ + K2 * FSrc2^ + K13 * FSrc3^);
        Inc(FSrc1);
        Inc(FSrc2);
        Inc(FSrc3);
        Inc(FDst);

        FDst^ := Trunc(K13 * FSrc1^ + K2 * FSrc2^ + K13 * FSrc3^);
        Inc(FSrc1);
        Inc(FSrc2);
        Inc(FSrc3);
        Inc(FDst);
      end;
    end;

    Result := True;
  except
    OutputDebugString('tsv: Error in TEdgeDetector.HorizontalEdgeRGB32 call...');
  end;
end;


function TEdgeDetector.HorVerEdgeRGB32(Data: Pointer; W, H: Integer; K13, K2: Double): Bool;
var
  U, I: Integer;
begin
  Result := False;
  try
    //If the picture is bigger than HD 1920 x 1080 recreate buffers
    if (W * H > 1920 * 1080) then
    begin
      FreeMemory(FBuffer1);
      FreeMemory(FBuffer2);

      FBuffer1 := GetMemory(W * H * 4);
      FBuffer2 := GetMemory(W * H * 4);
    end;

    //Horizontal
    for U := 0 to H - 1 do
    begin
      FSrc1 := PByte(Integer(Data) + U * W * 4);
      FSrc2 := PByte(Integer(Data) + U * W * 4 + 4);
      FSrc3 := PByte(Integer(Data) + U * W * 4 + 8);
      FDst  := PByte(Integer(FBuffer1) + U * W * 4 + 4);

      for I := 0 to W - 3 do
      begin
        FDst^ := Trunc(K13 * FSrc1^ + K2 * FSrc2^ + K13 * FSrc3^);
        Inc(FSrc1);
        Inc(FSrc2);
        Inc(FSrc3);
        Inc(FDst);

        FDst^ := Trunc(K13 * FSrc1^ + K2 * FSrc2^ + K13 * FSrc3^);
        Inc(FSrc1);
        Inc(FSrc2);
        Inc(FSrc3);
        Inc(FDst);

        FDst^ := Trunc(K13 * FSrc1^ + K2 * FSrc2^ + K13 * FSrc3^);
        Inc(FSrc1);
        Inc(FSrc2);
        Inc(FSrc3);
        Inc(FDst);

        FDst^ := Trunc(K13 * FSrc1^ + K2 * FSrc2^ + K13 * FSrc3^);
        Inc(FSrc1);
        Inc(FSrc2);
        Inc(FSrc3);
        Inc(FDst);
      end;
    end;

    //Vertical
    for U := 0 to H - 3 do
    begin
      FSrc1 := PByte(Integer(Data) + U * W * 4);
      FSrc2 := PByte(Integer(Data) + (U + 1) * W * 4);
      FSrc3 := PByte(Integer(Data) + (U + 2) * W * 4);
      FDst  := PByte(Integer(FBuffer2) + (U + 1) * W * 4);

      for I := 0 to W - 1 do
      begin
        FDst^ := Trunc(K13 * FSrc1^ + K2 * FSrc2^ + K13 * FSrc3^);
        Inc(FSrc1);
        Inc(FSrc2);
        Inc(FSrc3);
        Inc(FDst);

        FDst^ := Trunc(K13 * FSrc1^ + K2 * FSrc2^ + K13 * FSrc3^);
        Inc(FSrc1);
        Inc(FSrc2);
        Inc(FSrc3);
        Inc(FDst);

        FDst^ := Trunc(K13 * FSrc1^ + K2 * FSrc2^ + K13 * FSrc3^);
        Inc(FSrc1);
        Inc(FSrc2);
        Inc(FSrc3);
        Inc(FDst);

        FDst^ := Trunc(K13 * FSrc1^ + K2 * FSrc2^ + K13 * FSrc3^);
        Inc(FSrc1);
        Inc(FSrc2);
        Inc(FSrc3);
        Inc(FDst);
      end;
    end;

    //Combine Hor and Ver.
    FSrc1 := FBuffer1;
    FSrc2 := FBuffer2;
    FDst := Data;
    for I := 0 to W * H * 4 - 1 do
    begin
      FDst^ := Max(FSrc1^, FSrc2^);

      Inc(FSrc1);
      Inc(FSrc2);
      Inc(FDst);
    end;

    Result := True;
  except
    OutputDebugString('tsv: Error in TEdgeDetector.HorVerEdgeRGB32 call...');
  end;
end;

function TEdgeDetector.VerticalEdgeRGB32(Data: Pointer; W, H: Integer; K13, K2: Double): Bool;
var
  I, U: Integer;
begin
  Result := False;
  try
    //If the picture is bigger than HD 1920 x 1080 recreate buffers
    if (W * H > 1920 * 1080) then
    begin
      FreeMemory(FBuffer1);
      FBuffer1 := GetMemory(W * H * 4);
    end;

    CopyMemory(FBuffer1, Data, W * H * 4);

    for U := 0 to H - 3 do
    begin
      FSrc1 := PByte(Integer(FBuffer1) + U * W * 4);
      FSrc2 := PByte(Integer(FBuffer1) + (U + 1) * W * 4);
      FSrc3 := PByte(Integer(FBuffer1) + (U + 2) * W * 4);
      FDst  := PByte(Integer(Data) + (U + 1) * W * 4);

      for I := 0 to W - 1 do
      begin
        FDst^ := Trunc(K13 * FSrc1^ + K2 * FSrc2^ + K13 * FSrc3^);
        Inc(FSrc1);
        Inc(FSrc2);
        Inc(FSrc3);
        Inc(FDst);

        FDst^ := Trunc(K13 * FSrc1^ + K2 * FSrc2^ + K13 * FSrc3^);
        Inc(FSrc1);
        Inc(FSrc2);
        Inc(FSrc3);
        Inc(FDst);

        FDst^ := Trunc(K13 * FSrc1^ + K2 * FSrc2^ + K13 * FSrc3^);
        Inc(FSrc1);
        Inc(FSrc2);
        Inc(FSrc3);
        Inc(FDst);

        FDst^ := Trunc(K13 * FSrc1^ + K2 * FSrc2^ + K13 * FSrc3^);
        Inc(FSrc1);
        Inc(FSrc2);
        Inc(FSrc3);
        Inc(FDst);
      end;
    end;

    Result := True;
  except
    OutputDebugString('tsv: Error in TEdgeDetector.VerticalEdgeRGB32 call...');
  end;
end;

end.
