{*******************************************************}
{                                                       }
{                                                       }
{       Unit : PropEdit                                 }
{       Created by Tsviatko Jongov                      }
{       http://tsviatko.jongov.com                      }
{       tsviatko@jongov.com                             }
{                                                       }
{       Date : 10.11.2006                               }
{                                                       }
{       DirectShow video edge detect filter             }
{       property page.                                  }
{                                                       }
{*******************************************************}
unit PropEdit;

interface

uses
  Windows, SysUtils, BaseClass, DirectShow, ActiveX, main, Classes,
  Controls, ComCtrls, StdCtrls, ExtCtrls;

type
  TPropertiesForm = class(TFormPropertyPage)
    rgEdgeDetectType: TRadioGroup;
    lbSensitivity: TLabel;
    tbSensitivity: TTrackBar;
    lbMin: TLabel;
    lbMax: TLabel;
    lbK1: TLabel;
    edK1: TEdit;
    lbK2: TLabel;
    edK2: TEdit;
    lbK3: TLabel;
    edK3: TEdit;
    procedure FormDestroy(Sender: TObject);
    procedure rgEdgeDetectTypeClick(Sender: TObject);
    procedure tbSensitivityChange(Sender: TObject);
  private
    FInnerChange                : Bool;
    FEdgeDetectControl          : IEdgeDetectControl;
  public
    function OnConnect(Unknown: IUnknown): HRESULT; override;
    function OnDisconnect: HRESULT; override;
    function OnApplyChanges: HRESULT; override;
  end;


const
  CLSID_JongovEdgeDetectPropertyPage: TGUID = '{1BC4F1B7-5404-4D05-B746-B7605FF4CA42}';

var
  PropertiesForm: TPropertiesForm;

implementation

{$R *.DFM}

function TPropertiesForm.OnApplyChanges: HRESULT;
begin
  Result := E_FAIL;
  try

    Result := S_OK;
  except
    OutputDebugString('tsv: Error in TPropertiesForm.OnApplyChanges call...');
  end;
end;

function TPropertiesForm.OnConnect(Unknown: IInterface): HRESULT;
var
  EDT: Integer;
  K13, K2: Double;
begin
  Result := E_FAIL;
  try
    FInnerChange := True;
    try
      FEdgeDetectControl := Unknown as IEdgeDetectControl;
      FEdgeDetectControl.GetEdgeDetectType(EDT);
      case EDT of
        0, 1, 2, 4: rgEdgeDetectType.ItemIndex := EDT;
      else
        rgEdgeDetectType.ItemIndex := 0;
      end;

      FEdgeDetectControl.GetCoefficients(K13, K2);
    finally
      FInnerChange := False;
    end;

    tbSensitivity.Position := Abs(Trunc(K13 * 100));

    Result := S_OK;
  except
    OutputDebugString('tsv: Error in TPropertiesForm.OnConnect call...');
  end;
end;

function TPropertiesForm.OnDisconnect: HRESULT;
begin
  Result := E_FAIL;
  try
    Result := S_OK;
  except
    OutputDebugString('tsv: Error in TPropertiesForm.OnDisconnect call...');
  end;
end;

procedure TPropertiesForm.FormDestroy(Sender: TObject);
begin
  try
    FEdgeDetectControl := nil;
  except
    OutputDebugString('tsv: Error in TPropertiesForm.FormDestroy call...');
  end;
end;

procedure TPropertiesForm.rgEdgeDetectTypeClick(Sender: TObject);
begin
  try
    if FInnerChange then
      Exit;

    if FEdgeDetectControl <> nil then
    begin
      FEdgeDetectControl.SetEdgeDetectType(rgEdgeDetectType.ItemIndex);
    end;
  except
    OutputDebugString('tsv: Error in TPropertiesForm.rgEdgeDetectTypeClick call...');
  end;
end;

procedure TPropertiesForm.tbSensitivityChange(Sender: TObject);
var
  K13, K2: Double;
begin
  try
    if FInnerChange then
      Exit;
  
    K13 := -tbSensitivity.Position / 100; //from -0.01 to -1
    K2 := tbSensitivity.Position / 50;    //from  0.02 to  2

    if FEdgeDetectControl <> nil then
    begin
      FEdgeDetectControl.SetCoefficients(K13, K2);
    end;

    edK1.Text := Format('%.2f', [K13]);
    edK2.Text := Format('%.2f', [K2]);
    edK3.Text := Format('%.2f', [K13]);
  except
    OutputDebugString('tsv: Error in TPropertiesForm.tbSensitivityChange call...');
  end;
end;

initialization
  TBCClassFactory.CreatePropertyPage(TPropertiesForm, CLSID_JongovEdgeDetectPropertyPage);
end.
